//---------------------------------------------------------------
//
//  Ce code est librement inspiré de celui de Gabriel STOLTZ (Ecole des Ponts)
//
//---------------------------------------------------------------


//--------- options ---------
gradient_exact = 0;

ordre = 1;

//-- sinon calcul par differences finies pour eviter de coder le gradient --
hx = 0.01;
hy = 0.01;

//---------- point auquel on trace le plan tangent ------------

//point = [1,1];
point = [-sqrt(1.5),0];
//point=[2/3 sqrt(1-(2/3)^3)]

//------ pour les traces -------
dx = 0.1;
dy = 0.1;
Rx = 2.8;
Ry = 2.5;

//-------- fonction à tracer -------------

function z = V(x,y)
   z = (x^3 + y^2)*exp(-(x^2+y^2));
endfunction

function [grad] = nablaV(w)
  x = w(1);
  y = w(2);
  grad = zeros(2,1);
  grad(1) = 3*(x^2)*exp(-(x^2+y^2)) - 2*x*(x^3+y^2)*exp(-(x^2+y^2));
  grad(2) = 2*y*exp(-(x^2+y^2)) - 2*y*(x^3+y^2)*exp(-(x^2+y^2));
endfunction

function [hessienne] = HV(w)
x = w(1);
y = w(2);
hessienne = zeros(2,2);
hessienne(1,1) = exp(-(x^2+y^2)) * ( 6*x-14*x^3 -2*y^2 + 4*x^5 + 4*x^2*y^2) ;
hessienne(1,2) = exp(-(x^2+y^2)) * ( -6*x^2*y + 4*y*x^4 - 4*x*y + 4*x*y^3 );
hessienne(2,1) = hessienne(1,2);
hessienne(2,2) = exp(-(x^2+y^2)) * ( 2 - 10*y^2 - 2*x^3 + 4*y*x^3 + 4*y^3 ) ;
endfunction

//------- vecteur normal -------
if (gradient_exact == 0)
 normal_vector = nablaV(point);
else
 normal_vector = zeros(2,1);
 normal_vector(1) = ( V(point(1)+hx,point(2))-V(point(1)-hx,point(2)) )/(2*hx);
 normal_vector(2) = ( V(point(1),point(2)+hy)-V(point(1),point(2)-hy) )/(2*hy);
end
normal_vector

//hessienne

h = HV(point);

//---- definition du plan tangent -------

if (ordre == 1) then
deff('z = g(x,y)','z = V(point(1),point(2)) + normal_vector(1)*(x-point(1)) + normal_vector(2)*(y-point(2))');
elseif (ordre == 2) then
deff('z = g(x,y)','z = V(point(1),point(2)) + normal_vector(1)*(x-point(1)) + normal_vector(2)*(y-point(2))+ 0.5*h(1,1)*(x-point(1))^2 + h(1,2)*(x-point(1))*(y-point(2)) + 0.5*h(2,2)*(y-point(2))^2');
end

//--------- bornes -------

x = -Rx:dx:Rx;
y = -Ry:dy:Ry;

x1 = (point(1)-0.5):dx:(point(1)+0.5);
y1 = (point(2)-0.5):dy:(point(2)+0.5);

//-------- trace surface 3D -------------
 
z  = feval(x,y,V);
zz = feval(x1,y1,g);
// generation des facettes 3D à colorier
[X,Y,Z]    = genfac3d(x,y,z);   
[XX,YY,ZZ] = genfac3d(x1,y1,zz);
// 5 = couleur rouge pour toutes les facettes
plot3d(X, Y, list(Z, 5*ones(1,size(Z,2))));  
//mesh(X, Y, Z);  
e=gce(); e.color_mode=0;
// 2 = couleur bleue pour toutes les facettes
plot3d(XX,YY,list(ZZ,2*ones(1,size(ZZ,2))));  

//------ options de tracé ----------
a = gca(); // get current axes
a.rotation_angles = [76,-15];
a.box = "off";
a.axes_visible="off";

